'use strict';
var __awaiter =
	(this && this.__awaiter) ||
	function (thisArg, _arguments, P, generator) {
		function adopt(value) {
			return value instanceof P
				? value
				: new P(function (resolve) {
						resolve(value);
				  });
		}
		return new (P || (P = Promise))(function (resolve, reject) {
			function fulfilled(value) {
				try {
					step(generator.next(value));
				} catch (e) {
					reject(e);
				}
			}
			function rejected(value) {
				try {
					step(generator['throw'](value));
				} catch (e) {
					reject(e);
				}
			}
			function step(result) {
				result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected);
			}
			step((generator = generator.apply(thisArg, _arguments || [])).next());
		});
	};
var __importDefault =
	(this && this.__importDefault) ||
	function (mod) {
		return mod && mod.__esModule ? mod : { default: mod };
	};
Object.defineProperty(exports, '__esModule', { value: true });
const sqs_1 = __importDefault(require('aws-sdk/clients/sqs'));
class Lambda {
	constructor() {
		this.sns = (topicArn, processSnsMessage) => {
			const fn = (event) =>
				__awaiter(this, void 0, void 0, function* () {
					if (!Array.isArray(event.Records)) return;
					const record = event.Records[0];
					if (record.EventSource !== 'aws:sns' || record.Sns.TopicArn !== topicArn) return;
					yield processSnsMessage(record.Sns);
				});
			this.fns.push(fn);
		};
		this.sqs = (queueArn, processSqsRecord) => {
			const fn = (event) =>
				__awaiter(this, void 0, void 0, function* () {
					if (!Array.isArray(event.Records)) return;
					const records = event.Records;
					if (records[0].eventSource !== 'aws:sqs' || records[0].eventSourceARN !== queueArn) return;
					const fulfilledRecords = [];
					const errors = [];
					yield Promise.all(
						records.map((record) =>
							processSqsRecord(record)
								.then(() => {
									fulfilledRecords.push({
										Id: record.messageId,
										ReceiptHandle: record.receiptHandle
									});
								})
								.catch((err) => {
									errors.push(err);
								})
						)
					);
					if (errors.length) {
						if (fulfilledRecords.length) {
							yield this.sqsDeleteMessageBatch(queueArn, fulfilledRecords);
						}
						errors.forEach((err) => {
							console.error(err);
						});
						throw new Error(`SQS Batch Failure: ${errors.length} of ${records.length} failed`);
					}
				});
			this.fns.push(fn);
		};
		this.sqsFifo = (queueArn, processSqsRecord) => {
			const fn = (event) =>
				__awaiter(this, void 0, void 0, function* () {
					if (!Array.isArray(event.Records)) return;
					const records = event.Records;
					if (records[0].eventSource !== 'aws:sqs' || records[0].eventSourceARN !== queueArn) return;
					const fulfilledRecords = [];
					try {
						for (const record of records) {
							// eslint-disable-next-line no-await-in-loop
							yield processSqsRecord(record);
							fulfilledRecords.push({
								Id: record.messageId,
								ReceiptHandle: record.receiptHandle
							});
						}
					} catch (err) {
						// catch error and delete fulfilled record(s)
						// throw error to retry failed and remaining record(s)
						if (fulfilledRecords.length) {
							yield this.sqsDeleteMessageBatch(queueArn, fulfilledRecords);
						}
						console.error(err);
						const failedRecordsCount = records.length - fulfilledRecords.length;
						throw new Error(`SQS FIFO Batch Failure: ${failedRecordsCount} of ${records.length} failed`);
					}
				});
			this.fns.push(fn);
		};
		this.scheduledEvent = (ruleArn, processScheduledEvent) => {
			const fn = (event) =>
				__awaiter(this, void 0, void 0, function* () {
					if (event.source !== 'aws.events' || !event.resources.includes(ruleArn)) return;
					yield processScheduledEvent();
				});
			this.fns.push(fn);
		};
		this.eventBridge = (source, processEventBridge) => {
			const fn = (event) =>
				__awaiter(this, void 0, void 0, function* () {
					if (event.source !== source) return;
					yield processEventBridge(event.detail);
				});
			this.fns.push(fn);
		};
		this.handler = (event) =>
			__awaiter(this, void 0, void 0, function* () {
				for (const fn of this.fns) {
					const result = yield fn(event);
					if (result) {
						return result;
					}
				}
			});
		this.sqsDeleteMessageBatch = (queueArn, fulfilledRecords) =>
			__awaiter(this, void 0, void 0, function* () {
				const [, , , region, awsAccountId, queueName] = queueArn.split(':');
				const sqs = new sqs_1.default({ region });
				const result = yield sqs
					.getQueueUrl({
						QueueName: queueName,
						QueueOwnerAWSAccountId: awsAccountId
					})
					.promise();
				yield sqs.deleteMessageBatch({
					QueueUrl: result.QueueUrl,
					Entries: fulfilledRecords
				});
			});
		this.fns = [];
	}
}
exports.default = Lambda;
//# sourceMappingURL=index.js.map
